Package: DuplexDiscovereR
Title: Analysis of the data from RNA duplex probing experiments
Authors@R: 
        c(person("Egor", "Semenchenko", , 
        email = "yegor.smb@gmail.com", 
        role = c("aut", "cre","cph"),
        comment = c(ORCID = "0009-0007-5306-076X")),
        person("Volodymyr", "Tsybulskyi", ,
        email = "Volodymyr.Tsybulskyi@mdc-berlin.de", role = c("ctb"),
        comment = c(ORCID = "0009-0002-4141-6291")),
        person("Irmtraud M.", "Meyer", ,
        email = "irmtraud.m.meyer@cantab.net", role = c("aut","cph"),
             comment = c(ORCID = "0000-0002-4048-3479")))
Description: DuplexDiscovereR is a package designed for analyzing data
        from RNA cross-linking and proximity ligation protocols such as
        SPLASH, PARIS, LIGR-seq, and others. DuplexDiscovereR accepts
        input in the form of chimerically or split-aligned reads. It
        includes procedures for alignment classification, filtering,
        and efficient clustering of individual chimeric reads into
        duplex groups (DGs). Once DGs are identified, the package
        predicts RNA duplex formation and their hybridization energies.
        Additional metrics, such as p-values for random ligation
        hypothesis or mean DG alignment scores, can be calculated to
        rank final set of RNA duplexes. Data from multiple experiments
        or replicates can be processed separately and further compared
        to check the reproducibility of the experimental method.
License: GPL-3
URL: https://github.com/Egors01/DuplexDiscovereR/
BugReports: https://github.com/Egors01/DuplexDiscovereR/issues/
Encoding: UTF-8
NeedsCompilation: no
Version: 1.6.0
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BiocType: Software
biocViews: Sequencing, Transcriptomics, StructuralPrediction,
        Clustering, SplicedAlignment
Imports: Gviz, Biostrings, rtracklayer, GenomicAlignments,
        GenomicRanges, GenomeInfoDb, ggsci, igraph, rlang, scales,
        stringr, dplyr, tibble, tidyr, purrr, methods, grDevices,
        stats, utils, vctrs
Depends: R (>= 4.5), InteractionSet
LazyData: false
Suggests: knitr, UpSetR, BiocStyle, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
VignetteEngine: knitr
Config/testthat/edition: 3
Config/pak/sysreqs: cmake libglpk-dev make libbz2-dev libicu-dev
        libjpeg-dev liblzma-dev libpng-dev libuv1-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:03:52 UTC
RemoteUrl: https://github.com/bioc/DuplexDiscovereR
RemoteRef: RELEASE_3_23
RemoteSha: 6f2f2e9d95170bb05e2344d6385a940c45c98bbd
Packaged: 2026-04-29 21:04:38 UTC; root
Author: Egor Semenchenko [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0007-5306-076X>),
  Volodymyr Tsybulskyi [ctb] (ORCID:
    <https://orcid.org/0009-0002-4141-6291>),
  Irmtraud M. Meyer [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-4048-3479>)
Maintainer: Egor Semenchenko <yegor.smb@gmail.com>
Built: R 4.6.0; ; 2026-04-29 21:54:24 UTC; unix
