# Generated by roxygen2: do not edit by hand

export(DuplexDiscovererResults)
export(DuplexTrack)
export(annotateGI)
export(availableDisplayPars)
export(calculateLigationPvalues)
export(classifyTwoArmChimeras)
export(clusterDuplexGroups)
export(collapseIdenticalReads)
export(collapse_duplex_groups)
export(compareMultipleInteractions)
export(computeGISelfOverlaps)
export(convert_gi_to_ranges)
export(dd_get_chimeric_reads)
export(dd_get_chimeric_reads_stats)
export(dd_get_duplex_groups)
export(dd_get_reads_classes)
export(dd_get_run_stats)
export(getChimericJunctionTypes)
export(getRNAHybrids)
export(getSpliceJunctionChimeras)
export(get_char_count_cigar)
export(makeDfFromGi)
export(makeGiFromDf)
export(runDuplexDiscoPreproc)
export(runDuplexDiscoverer)
export(trimAroundJunction)
export(writeGiToSAMfile)
exportMethods(show)
import(Gviz)
import(InteractionSet)
import(dplyr)
import(grid)
import(rlang)
import(rtracklayer)
import(stringr)
import(tibble)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,letterFrequency)
importFrom(Biostrings,readBStringSet)
importFrom(GenomicAlignments,cigarWidthAlongReferenceSpace)
importFrom(GenomicRanges,GRanges)
importFrom(Gviz,availableDisplayPars)
importFrom(Gviz,getScheme)
importFrom(dplyr,first)
importFrom(ggsci,pal_igv)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb2hsv)
importFrom(igraph,V)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_louvain)
importFrom(igraph,decompose)
importFrom(igraph,edge.attributes)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,membership)
importFrom(igraph,simplify)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,getClass)
importFrom(methods,getClassDef)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(purrr,set_names)
importFrom(scales,rescale)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,setNames)
importFrom(tidyr,replace_na)
importFrom(tidyr,unite)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
importFrom(vctrs,vec_cast)
